@@ -1,18 +1,24 @@
 from koil.helpers import unkoil
 from koil.koil import Koil
 import inspect
+from typing import Callable, Type, TypeVar
+from koil.vars import current_loop
+
+T = TypeVar("T")
 
 
-def koilable(fieldname="__koil", add_connectors=False, **koilparams):
+def koilable(
+    fieldname: str = "__koil",
+    add_connectors: bool = False,
+    koil_class: Type[Koil] = Koil,
+    **koilparams
+) -> Callable[[Type[T]], Type[T]]:
     """
     Decorator to make an async generator koilable.
 
-
-
-
     """
 
-    def real_cls_decorator(cls):
+    def real_cls_decorator(cls: Type[T]) -> Type[T]:
 
         cls.__koilable__ = True
         assert hasattr(cls, "__aenter__"), "Class must implement __aenter__"
@@ -25,9 +31,11 @@ def koilable(fieldname="__koil", add_connectors=False, **koilparams):
         ), "__aexit__ must be a coroutine"
 
         def koiled_enter(self, *args, **kwargs):
-            setattr(self, fieldname, Koil(**koilparams))
-            getattr(self, fieldname).__enter__()
-            print(self.__aenter__)
+            if current_loop.get() is None:
+                if getattr(self, fieldname, None) is None:
+                    setattr(self, fieldname, koil_class(**koilparams))
+                getattr(self, fieldname).__enter__()
+
             return unkoil(self.__aenter__, *args, **kwargs)
 
         def koiled_exit(self, *args, **kwargs):
@@ -40,10 +48,19 @@ def koilable(fieldname="__koil", add_connectors=False, **koilparams):
             getattr(self, fieldname).__exit__(None, None, None)
             setattr(self, fieldname, None)
 
+        async def adisconnect(self):
+            return await self.__aexit__(None, None, None)
+
+        async def aconnect(self):
+            return await self.__aenter__()
+
         cls.__enter__ = koiled_enter
         cls.__exit__ = koiled_exit
-        cls.disconnect = disconnect
-        cls.connect = koiled_enter
+        if add_connectors:
+            cls.adisconnect = adisconnect
+            cls.aconnect = aconnect
+            cls.disconnect = disconnect
+            cls.connect = koiled_enter
 
         return cls
 
