@@ -4,6 +4,8 @@ import os
 import sys
 import threading
 from typing import Optional, Type
+
+from pydantic import BaseModel, Field
 from koil.errors import ContextError
 from koil.vars import *
 from koil.task import KoilGeneratorTask, KoilTask
@@ -54,6 +56,7 @@ def run_threaded_event_loop(loop):
                 task.cancel()
 
             async def gather():
+                logger.debug(f"Cancelling {tasks}")
                 await asyncio.gather(*tasks, return_exceptions=True)
 
             loop.run_until_complete(gather())
@@ -71,6 +74,7 @@ def run_threaded_event_loop(loop):
             if hasattr(loop, "shutdown_asyncgens"):
                 loop.run_until_complete(loop.shutdown_asyncgens())
         finally:
+            logger.debug("Loop Sucessfully Closed")
             loop.close()
 
 
@@ -89,12 +93,15 @@ def get_threaded_loop(name="KoilLoop", uvify=True):
 class Koil:
     def __init__(
         self,
+        *args,
         name="Koiloop",
         grace_period=None,
         uvify=True,
         task_class: Optional[Type[KoilTask]] = None,
         gen_class: Optional[Type[KoilGeneratorTask]] = None,
+        **kwargs,
     ) -> None:
+        super().__init__(*args, **kwargs)
         self.it = "it"
         self.name = name
         self.task_class = task_class
@@ -137,6 +144,7 @@ class Koil:
 
         self.popped_loop = get_threaded_loop(self.name, uvify=self.uvify)
         current_loop.set(self.popped_loop)
+        current_koil.set(self)
         return self
 
     async def aclose(self):
@@ -163,3 +171,4 @@ class Koil:
 
         current_taskclass.set(self.old_taskclass)
         current_taskclass.set(self.old_genclass)
+        current_koil.set(None)
